<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use App\Model\RechargeApi;
use Log;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;


class CustomerController extends Controller 
{

    private $rechargeApi;
    private $shop;
    private $shOptions;
    private $shApi;

    public function __construct() {

        $this->rechargeApi = new RechargeApi(); 

        // Create options for the API
        $this->shOptions = new Options();
        $this->shOptions->setVersion(env('SHOPIFY_STORE_API_VERSION', true));

        // Create the client and session
        $this->shApi = new BasicShopifyAPI($this->shOptions);
        $this->shApi->setSession(new Session(env('SHOPIFY_STORE_DOMAIN', true), env('SHOPIFY_STORE_PASSWORD', true)));
        //$this->shop = Auth::user();
    }

    public function home() {

        return view('welcome');
    }

    /**
     * Get primary customer account details
     */
    public function processWithCustomerDetails(Request $request) {

        $errorMessage = '';
        $emailAddress = '';
        $accountEmails = [];
        $customerId = !empty($request->get('id')) ? $request->get('id') : '';
        if (empty($customerId)) {

            $errorMessage = 'Failed to get the customer details from Recharge';
            $error = "Failed to get the customerId from the URL";
            Log::channel('MergeAccountLog')->info($error);
            return view('customer',compact('errorMessage'));
        }
        $customerDetails = $this->rechargeApi->getCustomerDetailsByShopifyCustomerId($customerId);

        if (!empty($customerDetails->error) || !empty($customerDetails->errors) || 
        empty($customerDetails) || empty($customerDetails->customers)) {

            // Log Entry
            $error  = "Unknown error";
            if (!empty($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (!empty($customerDetails->errors)) {

                $error = json_encode($customerDetails->errors);
            } else if (empty($customerDetails) || empty($customerDetails->customers)) {

                $error = 'Empty response from the API';
            }
            $errorMessage = 'The primary account has no subscription metafield';
            $rechargeError = "Failed to get the customer details from Recharge - Error: ".$error;
            Log::channel('MergeAccountLog')->info(
                "Failed to get the customer details from recharge: ".
                " | CustomerId -> ".$customerId.
                " | Error -> ".$rechargeError
            );
            $accountEmails = [
                "customerId" => '',
                "primaryEmail" => '',
                "errorMessage" => $errorMessage
            ];
            return view('customer',compact('accountEmails'));
        }
        $emailAddress = $customerDetails->customers[0]->email;
        $customerSearTag = 'accountMerged:'.$customerId;

        $shopifyCustomerDetails = $this->shApi->rest('GET',
        '/admin/customers/search.json',["query" => ["customer_tag" => $customerSearTag]]);

        $shCustomerDetails = [];
        $shCustomerDetails = isset($shopifyCustomerDetails['body']['customers']) ? 
        $shopifyCustomerDetails['body']['container']['customers'] : '';

        if (!empty($shCustomerDetails['error']) || !empty($shCustomerDetails['errors'])) {

            // Log Entry
            $shError  = "Unknown error";
            if (!empty($shCustomerDetails['error'])) {

                $shError = $shCustomerDetails['error'];

            } else if (!empty($shCustomerDetails['errors'])) {

                $shError = json_encode($shCustomerDetails['errors']);
            } 

            $errorMessage = 'Failed to get the customer details from Shopify';
            $shErrorMessage = "Failed to get the customer details from Shopify with tagged (".$customerSearTag.") - Error: ".$shError;
            Log::channel('MergeAccountLog')->info(
                "Failed to get the customer details from Shopify: ".
                " | EmailAddress -> ".$requestEmailAddress.
                " | Error -> ".$shErrorMessage
            );
            return view('customer',compact('errorMessage'));
        }

        $accountEmails = [
            "primaryEmail" => $emailAddress,
            "customerId" => $customerId
        ];
        $accountEmails["additionalEmails"] = [];
        if (isset($shCustomerDetails) || !empty($shCustomerDetails)) {
            foreach ($shCustomerDetails as $shKey => $shValue) {

                $accountEmails["additionalEmails"][] = $shValue['email'];
            }
            $accountEmails["additionalEmails"] = array_reverse($accountEmails["additionalEmails"]);
        }

        return view('customer',compact('accountEmails'));
    }

    /**
     * Copy the primary account subscription meta to the additional email
     */
    public function addAdditionalEmailWithPrimaryAccount(Request $request) {

        $receivedData = !empty($request->post('data')) ? $request->post('data') : '';
        $requestEmailAddress = !empty($receivedData['email']) ? trim($receivedData['email']) : '';
        $primaryCustomerId = !empty($receivedData['customerId']) ? trim($receivedData['customerId']) : '';

        // $additionalEmailsAdded = [];
        // $additionalEmailsAdded = [
        //     "additionalEmail" => $requestEmailAddress,
        //     "message" => 'Successfully added the email to the primary account',
        // ];
        // $response = array(
        //     "status"  => 200,
        //     "message" => "Success",
        //     "body"   => $additionalEmailsAdded
        // );
        // echo json_encode($response);
        // die();

        if (empty($requestEmailAddress) || empty($primaryCustomerId)) {

            $response = [];
            $error = 'The email address or the primary account customerId is empty';
            Log::channel('MergeAccountLog')->info($error);
            $response = array(
                "status"  => 600,
                "message" => "Failed",
                "body"   => [
                    "error" => 'Failed to add the additional account to the primary account'
                ]
            );
            echo json_encode($response);
            die();
        }

        $customerSearchResult = $this->shApi->rest('GET',
        '/admin/customers/search.json',["query" => ["email" => $requestEmailAddress]]);

        $searchResult = [];
        $searchResult = isset($customerSearchResult['body']['customers']) ? 
        $customerSearchResult['body']['container']['customers'] : '';

        if (!empty($searchResult['error']) || !empty($searchResult['errors']) || 
        empty($searchResult)) {

            // Log Entry
            $error  = "Unknown error";
            if (!empty($searchResult['error'])) {

                $error = $searchResult['error'];

            } else if (!empty($searchResult['errors'])) {

                $error = json_encode($searchResult['errors']);
            } else if (empty($searchResult)) {

                $error = 'Empty response from the API';
            }

            $response = [];
            $error = "Failed to get the customer details that tried to added to the primary account from Shopify - Error: ".$error;
            Log::channel('MergeAccountLog')->info(
                "Failed to get the customer details from Shopify: ".
                " | EmailAddress -> ".$requestEmailAddress.
                " | Error -> ".$error
            );
            $response = array(
                "status"  => 600,
                "message" => "Failed",
                "body"   => [
                    "error" => 'Failed to get the customer details of '.$requestEmailAddress.' from Shopify'
                ]
            );
            echo json_encode($response);
            die();
        }

        $requestedStrToLowerEmail = strtolower($requestEmailAddress);
        $searchedEmailAddress = '';
        $searchedCustomerId = '';
        $searchedCustomerTag = '';
        foreach ($searchResult as $customerKey => $customerValue) {

            $searchedEmailAddress = !empty($customerValue['email']) ? trim(strtolower($customerValue['email'])) : '';
            if ($searchedEmailAddress == $requestedStrToLowerEmail) {

                $searchedCustomerId = $customerValue['id'];
                $searchedCustomerTag = $customerValue['tags'];
            }
        }

        if (empty($searchedCustomerId)) {

            $error = 'Failed to find the '.$requestEmailAddress.' in Shopify';
            Log::channel('MergeAccountLog')->info($error);
            $response = array(
                "status"  => 600,
                "message" => "Failed",
                "body"   => [
                    "error" => 'Failed to get the customer details of '.$requestEmailAddress.' from Shopify'
                ]
            );
            echo json_encode($response);
            die();
        }
        $emailAccountMergedStatus = 0;
        $mergedTag = '';
        if (!empty($searchedCustomerTag)) {

            $explodedTags = explode(",", $searchedCustomerTag);
            $tagArr = array_map("trim", $explodedTags);
            foreach ($tagArr as $tagKey => $tagValue) {

                if (strpos($tagValue, "accountMerged") !== false) {

                    $emailAccountMergedStatus = 1;
                    $mergedTag = $tagValue;
                }
            }

            if ($emailAccountMergedStatus == 1) {

                $error = "The account is already merged with a primary account, CustomerMergedTag:".$mergedTag;
                Log::channel('MergeAccountLog')->info($error);
                $response = array(
                    "status"  => 600,
                    "message" => "Failed",
                    "body"   => [
                        "error" => 'The account is already merged with an account'
                    ]
                );
                echo json_encode($response);
                die();
            }
        }

        $additionalAccountMetafieldDetails = $this->shApi->rest('GET',
        '/admin/customers/'.$searchedCustomerId.'/metafields.json');

        $additionalAccMetaDetails = [];
        $additionalAccMetaDetails = isset($additionalAccountMetafieldDetails['body']['metafields']) ? 
        $additionalAccountMetafieldDetails['body']['container']['metafields'] : '';

        $addMetafieldValue = '';
        $metaCreateRequest = [];

        foreach ($additionalAccMetaDetails as $addMetafields) {
            if (trim($addMetafields['key']) == 'customer_string' && trim($addMetafields['namespace']) == 'subscriptions') {
                $addMetafieldValue = $addMetafields['value'];
                break;
            } else {
                continue;
            }
        }

        if (!empty($addMetafieldValue)) {

            $error = "The entered email account has already a subscription metafield, Email:".$requestEmailAddress;
            Log::channel('MergeAccountLog')->info($error);
            $response = array(
                "status"  => 600,
                "message" => "Failed",
                "body"   => [
                    "error" => 'The entered email account has already a subscription metafield'
                ]
            );
            echo json_encode($response);
            die();
        }

        $primaryAccountMetafieldDetails = $this->shApi->rest('GET',
        '/admin/customers/'.$primaryCustomerId.'/metafields.json');

        $primaryAccMetaDetails = [];
        $primaryAccMetaDetails = isset($primaryAccountMetafieldDetails['body']['metafields']) ? 
        $primaryAccountMetafieldDetails['body']['container']['metafields'] : '';

        $metafieldValue = '';
        $metaCreateRequest = [];

        foreach ($primaryAccMetaDetails as $metafields) {
            if (trim($metafields['key']) == 'customer_string' && trim($metafields['namespace']) == 'subscriptions') {
                $metafieldValue = $metafields['value'];
                break;
            } else {
                continue;
            }
        }

        if (empty($metafieldValue)) {

            $error = "The subscription metafield is not found in the primary account, Email:".$requestEmailAddress;
            Log::channel('MergeAccountLog')->info($error);
            $response = array(
                "status"  => 600,
                "message" => "Failed",
                "body"   => [
                    "error" => 'The subscription metafield is not found in the primary account'
                ]
            );
            echo json_encode($response);
            die();
        }

        $metaParams = ["metafield" => ["namespace" => "subscriptions", "key" => "customer_string", "value" => $metafieldValue, "value_type" => "string"]];
        $metaCreateResponse = $this->shApi->rest('POST', '/admin/customers/'.$searchedCustomerId.'/metafields.json',$metaParams);
        
        if (!empty($metaCreateResponse['error']) || !empty($metaCreateResponse['errors']) || 
        empty($metaCreateResponse)) {

            // Log Entry
            $error  = "Unknown error";
            if (!empty($metaCreateResponse['error'])) {

                $error = $metaCreateResponse['error'];

            } else if (!empty($metaCreateResponse['errors'])) {

                $error = json_encode($metaCreateResponse['errors']);
            } else if (empty($metaCreateResponse)) {

                $error = 'Empty response from the API';
            }

            $error = "Failed to create the subscription metafield in the new account (Email:".$requestEmailAddress.") - Error: ".$error;
            Log::channel('MergeAccountLog')->info(
                "Failed to create the subscription metafield in the new account: ".
                " | EmailAddress -> ".$requestEmailAddress.
                " | Error -> ".$error
            );
            $response = array(
                "status"  => 600,
                "message" => "Failed",
                "body"   => [
                    "error" => 'Failed to merge the customer to the primary account'
                ]
            );
            echo json_encode($response);
            die();
        }

        $mergedTagToTheCustomer = "accountMerged:".$primaryCustomerId;
        $tagToUpdate = $searchedCustomerTag.",".$mergedTagToTheCustomer;

        $params = ["customer"=>["id" => $searchedCustomerId, "tags" => $tagToUpdate]];
        $customerAccountUpdatedInfo = $this->shApi->rest('PUT',
        '/admin/customers/'.$searchedCustomerId.'.json',$params);

        if (!empty($customerAccountUpdatedInfo['error']) || !empty($customerAccountUpdatedInfo['errors']) || 
        empty($customerAccountUpdatedInfo)) {

            // Log Entry
            $error  = "Unknown error";
            if (!empty($customerAccountUpdatedInfo['error'])) {

                $error = $customerAccountUpdatedInfo['error'];

            } else if (!empty($customerAccountUpdatedInfo['errors'])) {

                $error = json_encode($customerAccountUpdatedInfo['errors']);
            } else if (empty($customerAccountUpdatedInfo)) {

                $error = 'Empty response from the API';
            }

            $error = "Failed to add the additional email to the primary account - Error: ".$error;
            Log::channel('MergeAccountLog')->info(
                "Failed to add the additional email to the primary account: ".
                " | EmailAddress -> ".$requestEmailAddress.
                " | Error -> ".$error
            );
            $response = array(
                "status"  => 600,
                "message" => "Failed",
                "body"   => [
                    "error" => 'Failed to add the additional email to the primary account'
                ]
            );
            echo json_encode($response);
            die();
        }

        $additionalEmailsAdded = [];
        $additionalEmailsAdded = [
            "additionalEmail" => $requestEmailAddress,
            "message" => 'Successfully added the email to the primary account',
        ];
        $response = array(
            "status"  => 200,
            "message" => "Success",
            "body"   => $additionalEmailsAdded
        );
        echo json_encode($response);
    }

}
